/**
 * @file application.hpp
 * @brief Application class
 *
 * @author Eric Butler (edbutler)
 */

#ifndef _462_APPLICATION_APPLICATION_HPP_
#define _462_APPLICATION_APPLICATION_HPP_

#include "math/math.hpp"
#include <string>

namespace _462 {

class Window;

class Application
{
public:

    enum ModifierKey {
        MODIFIER_NONE = 0,
        MODIFIER_CTRL = 1 << 1,
        MODIFIER_SHIFT = 1 << 2,
        MODIFIER_ALT = 1 << 3
    };

    enum MouseButton {
        MOUSE_LEFT = 0,
        MOUSE_MIDDLE = 1,
        MOUSE_RIGHT = 2
    };

    virtual ~Application() { }

    /**
     * Initializes the application.
     * @return true on successful initialization, false otherwise.
     */
	virtual bool initialize( Window* window, int argc, char* argv[] ) = 0;

    /**
     * Cleans up any resources used by the application.
     */
	virtual void destroy( Window* window ) = 0;

    /**
     * Updates the application for the current frame.
     */
	virtual void update( Window* window, real_t time ) = 0;

    /**
     * Renders the application for the current frame.
     */
	virtual void render( Window* window ) = 0;

    virtual void on_mouse_press( Window* window, MouseButton button, int x, int y, size_t modifiers, bool pressed ) { }

    virtual void on_key_press( Window* window, char key, size_t modifiers, bool pressed ) { }

    virtual void on_mouse_move( Window* window, int x, int y ) { }
};

class Window
{
public:

    typedef void ( *MenuCallback )( Application* app, Window* window );

    /**
     * Starts the application and returns a window object.
     * Does not return. Creates the window instance.
     */
    static void start_application( Application* app,
                                   int argc, char *argv[],
                                   int width, int height, real_t fps,
                                   const char* title );

    void get_dimension( int* width, int* height );

    void add_menu_item( const char* name, MenuCallback callback );

    void terminate_application();

private:

    Window();

    ~Window();

};

} /* _462 */

#endif /* _462_APPLICATION_APPLICATION_HPP_ */

