/**
 * @file model.hpp
 * @brief Model class
 *
 * @author Eric Butler (edbutler)
 */

#ifndef _462_SCENE_MODEL_HPP_
#define _462_SCENE_MODEL_HPP_

#include "math/vector.hpp"
#include "math/quaternion.hpp"
#include <vector>
#include <cassert>

namespace _462 {

/**
 * A mesh of triangles.
 */
class Model
{
public:

    struct Vertex
    {
        // the position of the vertex
        Vector3 position;
        // the texture coordinate of the vertex
        Vector2 tex_coord;
    };

    struct Triangle
    {
        size_t vertices[3];
    };

    typedef std::vector< Triangle > TriangleList;
    typedef std::vector< Vertex > VertexList;

    const VertexList& get_vertices()
    {
        return vertices;
    }

    const TriangleList& get_triangles()
    {
        return triangles;
    }

    /**
     * @return true on success, false on failure.
     */
    bool load_from_obj( const char* filename );

private:

    TriangleList triangles;
    VertexList vertices;

};

} /* _462 */

#endif /* _462_SCENE_MODEL_HPP_ */

